import {
  __spreadProps,
  __spreadValues,
  presets
} from "./chunk-OBIMVIXS.mjs";

// src/index.ts
import { promises as fs } from "fs";
import { createUnplugin } from "unplugin";
import { throttle } from "@antfu/utils";

// src/core/options.ts
import { resolve } from "path";
import { toArray } from "@antfu/utils";
import { createFilter } from "@rollup/pluginutils";
import hasPkg from "has-pkg";
function resolveOptions(options = {}) {
  const imports = flattenImportsMap(options.imports, options.presetOverriding);
  const { dts = hasPkg("typescript") } = options;
  const resolved = __spreadProps(__spreadValues({
    sourceMap: false,
    resolvedImports: {},
    presetOverriding: false
  }, options), {
    dts: dts === false ? false : dts === true ? resolve("auto-imports.d.ts") : resolve(dts),
    imports,
    resolvers: toArray(options.resolvers),
    idFilter: createFilter(options.include || [/\.[jt]sx?$/, /\.vue\??/, /\.svelte$/], options.exclude || [/node_modules/, /\.git/])
  });
  return resolved;
}
function flattenImportsMap(map, overriding = false) {
  const flat = {};
  toArray(map).forEach((definition) => {
    if (typeof definition === "string") {
      if (!presets[definition])
        throw new Error(`[auto-import] preset ${definition} not found`);
      const preset = presets[definition];
      definition = typeof preset === "function" ? preset() : preset;
    }
    for (const mod of Object.keys(definition)) {
      for (const id of definition[mod]) {
        const meta = {
          module: mod
        };
        if (Array.isArray(id)) {
          meta.name = id[1];
          meta.from = id[0];
        } else {
          meta.name = id;
        }
        if (flat[meta.name] && !overriding)
          throw new Error(`[auto-import] identifier ${meta.name} already defined with ${flat[meta.name].module}`);
        flat[meta.name] = meta;
      }
    }
  });
  return flat;
}

// src/core/transform.ts
import MagicString from "magic-string";
var excludeRE = [
  /\bimport\s*([\w_$]*?),?\s*\{([\s\S]*?)\}\s*from\b/g,
  /\bfunction\s*([\s\S]+?)\s*\(/g,
  /\b(?:const|let|var)\s*([\w\d_$]+?)\b/g
];
var matchRE = /\b(\w+)\b/g;
var importAsRE = /^.*\sas\s+/;
var multilineCommentsRE = /\/\*(.|[\r\n])*?\*\//gm;
var singlelineCommentsRE = /\/\/.*/g;
function stripeComments(code) {
  return code.replace(multilineCommentsRE, "").replace(singlelineCommentsRE, "");
}
function transform(code, id, {
  imports,
  sourceMap,
  resolvers,
  resolvedImports = {}
}) {
  const noComments = stripeComments(code);
  const identifiers = new Set(Array.from(noComments.matchAll(matchRE)).map((i) => i[1]));
  if (!identifiers.size)
    return null;
  for (const regex of excludeRE) {
    Array.from(noComments.matchAll(regex)).flatMap((i) => {
      var _a, _b;
      return [...((_a = i[1]) == null ? void 0 : _a.split(",")) || [], ...((_b = i[2]) == null ? void 0 : _b.split(",")) || []];
    }).map((i) => i.replace(importAsRE, "").trim()).forEach((i) => identifiers.delete(i));
  }
  if (!identifiers.size)
    return null;
  const modules = {};
  Array.from(identifiers).forEach((name) => {
    let info = resolvedImports[name] || imports[name];
    if (!info && (resolvers == null ? void 0 : resolvers.length)) {
      const resolved = firstNonNullResult(resolvers, name);
      if (resolved) {
        if (typeof resolved === "string") {
          info = {
            module: resolved,
            name,
            from: "default"
          };
        } else {
          info = resolved;
        }
        resolvedImports[name] = info;
      }
    }
    if (!info)
      return;
    if (!modules[info.module])
      modules[info.module] = [];
    modules[info.module].push(info);
  });
  if (!Object.keys(modules).length)
    return;
  const importStatements = Object.entries(modules).map(([moduleName, names]) => {
    const imports2 = names.map(({ name, from }) => from ? `${from} as ${name}` : name).join(", ");
    return `import { ${imports2} } from '${moduleName}';`;
  }).join("");
  const s = new MagicString(code);
  s.prependLeft(0, importStatements);
  return {
    code: s.toString(),
    map: sourceMap ? s.generateMap() : null
  };
}
function firstNonNullResult(array, name) {
  for (let i = 0; i < array.length; i++) {
    const res = array[i](name);
    if (res)
      return res;
  }
}

// src/core/dts.ts
function generateDeclration(imports, resolvedImports = {}) {
  const body = [
    ...Object.entries(imports),
    ...Object.entries(resolvedImports)
  ].map(([name, info]) => `  const ${name}: typeof import('${info.module}')['${info.from || name}']`).join("\n");
  return `// Generated by 'unplugin-auto-import'
// We suggest you to commit this file into your source control
declare global {
${body}
}
export {}
`;
}

// src/index.ts
var src_default = createUnplugin((options) => {
  const resolved = resolveOptions(options);
  if (!Object.keys(resolved.imports).length && !resolved.resolvers.length)
    console.warn("[auto-import] plugin installed but no imports has defined, see https://github.com/antfu/unplugin-auto-import#configurations for configurations");
  const generateDeclaration = throttle(500, false, () => {
    if (!resolved.dts)
      return;
    fs.writeFile(resolved.dts, generateDeclration(resolved.imports, resolved.resolvedImports), "utf-8");
  });
  generateDeclaration();
  return {
    name: "unplugin-auto-import",
    enforce: "post",
    transformInclude(id) {
      return resolved.idFilter(id);
    },
    transform(code, id) {
      const res = transform(code, id, resolved);
      if (res && resolved.resolvers.length)
        generateDeclaration();
      return res;
    }
  };
});

export {
  src_default
};
